@@ -34,9 +34,9 @@ int cidr_parse(const char* cidr, struct cidr_spec* spec)
 		return 0;	/* invalid CIDR */
 	if ((CIDR[0] > 255) || (CIDR[1] > 255) || (CIDR[2] > 255) || (CIDR[3] > 255))
 		return 0;	/* invalid CIDR */
-		
+
 	spec->mask_32 = ~((1 << (32-maskbits))-1) & 0xFFFFFFFF;	/* requires 32-bit int or larger */
-	spec->cidr_32 = (CIDR[0] << 24) | (CIDR[1] << 16) | (CIDR[2] << 8) | CIDR[3];
+	spec->cidr_32 = ((CIDR[0] << 24) | (CIDR[1] << 16) | (CIDR[2] << 8) | CIDR[3]) & spec->mask_32;
 	
 	return 1;	/* success */
 }
@@ -48,19 +48,28 @@ int cidr_parse(const char* cidr, struct cidr_spec* spec)
 */
 int cidr_match(const char* ip, struct cidr_spec* spec)
 {
-	unsigned int IP[4];	/* 4 octets for IP address */
-	unsigned int IP_32;	/* 32-bit version of IP address */
-
 	if (!ip || !spec)
 		return 0;
-		
+	/* See if IP matches */
+	return ((ip_to_int(ip) & spec->mask_32) == spec->cidr_32);
+}
+
+
+
+/*
+	Convert IP address string to 32-bit integer version
+	Returns 0 on failure
+*/
+unsigned int ip_to_int(const char* ip)
+{
+	unsigned int IP[4];	/* 4 octets for IP address */
+
+	if (!ip)
+		return 0;
 	/* Parse the supplied IP address */
 	if (sscanf(ip, "%u.%u.%u.%u", &IP[0], &IP[1], &IP[2], &IP[3]) != 4)
-		return 0; 	/* invalid IP */
+		return 0;	/* invalid IP */
 	if ((IP[0] > 255) || (IP[1] > 255) || (IP[2] > 255) || (IP[3] > 255))
 		return 0;	/* invalid IP */
-		
-	/* See if IP matches */
-	IP_32 = (IP[0] << 24) | (IP[1] << 16) | (IP[2] << 8) | IP[3];
-	return ((IP_32 & spec->mask_32) == (spec->cidr_32 & spec->mask_32));
+	return (IP[0] << 24) | (IP[1] << 16) | (IP[2] << 8) | IP[3];
 }
