@@ -1,5 +1,5 @@
 /*
-  grepcidr 1.1 - Filter IP addresses matching IPv4 CIDR specification
+  grepcidr 1.2 - Filter IP addresses matching IPv4 CIDR specification
   Copyright (C) 2004  Jem E. Berkes <jberkes@pc-tools.net>
 
   This program is free software; you can redistribute it and/or modify
@@ -26,18 +26,19 @@
 #include "cidr.h"
 #include "getopt.h"
 
-#define TXT_VERSION	"grepcidr 1.1, by Jem Berkes <jberkes@pc-tools.net>"
+#define TXT_VERSION	"grepcidr 1.2\nCopyright (C) 2004  Jem E. Berkes <jberkes@pc-tools.net>\n"
 #define TXT_USAGE	"Usage: grepcidr [-V] [-c] [-v] [-e PATTERN | -f FILE]\n"
 #define MAXFIELD	512
 #define TOKEN_SEPS	"\t ,\r\n"
 
 
-unsigned int counting = 0;		/* when non-zero, counts matches */
-int invert = 0;
-int patcount = 0;			/* number of elements in patterns */
-struct cidr_spec** patterns = NULL;	/* array of cidr_spec's */
-char* filename = NULL;
 static char shortopts[] = "ce:f:vV";
+char* filename = NULL;
+int counting = 0;			/* when non-zero, counts matches */
+int invert = 0;				/* flag for inverted mode */
+int total_patterns = 0;			/* total CIDR patterns */
+int pattern_count[8];			/* pattern counts in each bucket */
+struct cidr_spec* bucket[8];		/* pattern buckets */
 
 
 /*
@@ -46,6 +47,7 @@ static char shortopts[] = "ce:f:vV";
 */
 int main(int argc, char* argv[])
 {
+	unsigned int bucketno;
 	char* patbuf = NULL;
 	char line[MAXFIELD];
 	int foundopt;
@@ -129,12 +131,9 @@ int main(int argc, char* argv[])
 		}
 	}
 	
-	/* Convert big list in patbuf to indexed list, via tokenized patterns */
+	/* Fill elements from big list (patbuf) into appropriate buckets */
 	{
-		char *entire, *token;
-		int curpat = 0;
-		
-		entire = malloc(1 + strlen(patbuf));
+		char *token, *entire = malloc(1 + strlen(patbuf));
 		strcpy(entire, patbuf);
 		if (!filename)
 		{
@@ -145,46 +144,68 @@ int main(int argc, char* argv[])
 		token = strtok(patbuf, TOKEN_SEPS);
 		while (token)
 		{
-			patcount++;
+			total_patterns++;
 			token = strtok(NULL, TOKEN_SEPS);
 		}
 		free(patbuf);
-		patterns = calloc(patcount, sizeof(struct cidr_spec*));
+		for (bucketno=0; bucketno<8; bucketno++)
+			bucket[bucketno] = calloc(total_patterns, sizeof(struct cidr_spec));
 		token = strtok(entire, TOKEN_SEPS);
 		while (token)
 		{
-			patterns[curpat] = malloc(sizeof(struct cidr_spec));
-			if (!cidr_parse(token, patterns[curpat]))
-			{
+			struct cidr_spec spec;
+			if (!cidr_parse(token, &spec))
 				fprintf(stderr, "Invalid CIDR %s\n", token);
-				free(patterns[curpat]);
-				patterns[curpat] = NULL;	/* disable entry */
+			else
+			{
+				struct cidr_spec* patterns;
+				if (spec.mask_32 < 0xE0000000)
+				{
+					/* Rare, must put in all buckets */
+					for (bucketno=0; bucketno<8; bucketno++)
+					{
+						patterns = bucket[bucketno];
+						patterns[pattern_count[bucketno]++] = spec;
+					}
+				}
+				else
+				{
+					/* Determine and place in appropriate bucket*/
+					bucketno = (spec.cidr_32 >> 29) & 0x00000007;
+					patterns = bucket[bucketno];
+					patterns[pattern_count[bucketno]++] = spec;
+				}
 			}
-			curpat++;
 			token = strtok(NULL, TOKEN_SEPS);
 		}
 		free(entire);
 	}
-
-
+	
 	/* Do line matching */
 	while (fgets(line, sizeof(line), stdin))
 	{
-		int curpat, match=0;
-		for (curpat=0; (curpat<patcount) && !match; curpat++)
-			match = cidr_match(line, patterns[curpat]);
-
-		if (invert ^ match)
+		unsigned int IP_32 = ip_to_int(line);
+		bucketno = (IP_32 >> 29) & 0x00000007;
+		if (IP_32)
 		{
-			if (counting)
-				counting++;
-			else
-				printf("%s", line);
+			int curpat, match=0;
+			struct cidr_spec* patterns = bucket[bucketno];
+
+			for (curpat=0; (curpat<pattern_count[bucketno]) && !match; curpat++)
+				match = ((IP_32 & patterns[curpat].mask_32) == patterns[curpat].cidr_32);
+			if (invert ^ match)
+			{
+				if (counting)
+					counting++;
+				else
+					printf("%s", line);
+			}
 		}
 	}
-	
+
 	if (counting)
 		printf("%u\n", counting-1);
-	free(patterns);
+	for (bucketno=0; bucketno<8; bucketno++)
+		free(bucket[bucketno]);
 	return 0;	
 }
